import gatt
import threading
import paho.mqtt.client as mqtt
import datetime

manager = gatt.DeviceManager(adapter_name='hci1')

class MyMQTTClass(mqtt.Client):

    def __init__(self):
        self.device = None
        super(MyMQTTClass, self).__init__()

    def __init__(self, device):
        self.device = device
        super(MyMQTTClass, self).__init__()

    def on_connect(self, mqttc, obj, flags, rc):
        print("rc: "+str(rc))

    def on_message(self, mqttc, obj, msg):
        print(msg.topic+" "+str(msg.qos)+" "+str(msg.payload))
        self.device.sendData(str(msg.payload))

    def on_publish(self, mqttc, obj, mid):
        print("mid: "+str(mid))

    def on_subscribe(self, mqttc, obj, mid, granted_qos):
        print("Subscribed: "+str(mid)+" "+str(granted_qos))

    def on_log(self, mqttc, obj, level, string):
        print(string)

class AnyDevice(gatt.Device):
    def services_resolved(self):
        super().services_resolved()
        self.fb_service = next(
            s for s in self.services
            if s.uuid == '6e400001-b5a3-f393-e0a9-e50e24dcca9e')
        self.tx_characteristic = next(
            c for c in self.fb_service.characteristics
            if c.uuid == '6e400002-b5a3-f393-e0a9-e50e24dcca9e')
        print('resolve ok')
        self.tx_characteristic.write_value(bytearray("ECHO", 'utf-8'))

    def characteristic_value_updated(self, characteristic, value):
        print(str(characteristic)+", value: "+value.decode("utf-8"))

    def characteristic_write_value_succeeded(self, characteristic):
        print("Write successful.")

    def characteristic_write_value_failed(self, characteristic, error):
        print("Write failed. "+str(error))

    def sendData(self, payload):
        print('sending data over UART')
        self.fb_service = next(
            s for s in self.services
            if s.uuid == '6e400001-b5a3-f393-e0a9-e50e24dcca9e')
        self.tx_characteristic = next(
            c for c in self.fb_service.characteristics
            if c.uuid == '6e400002-b5a3-f393-e0a9-e50e24dcca9e')
        self.tx_characteristic.write_value(bytearray(payload, 'utf-8'))

class Manager(gatt.DeviceManager):
   def run(self):
      print("Running Manager")
      super().run()
   def quit(self):
      print("Stopping Manager")
      super().stop()  

manager = Manager(adapter_name = 'hci1')
device = AnyDevice(mac_address='dd:ea:e8:6b:39:8d', manager=manager)
thread1 = threading.Thread(target = manager.run)

mqttc = MyMQTTClass(device)
mqttc.username_pw_set(username="kitty",password="kitto")
mqttc.connect("127.0.0.1", 1883, 60)
mqttc.subscribe("FLMS/FB/FBDDEAE86B398D/commands", 0)

thread2 = threading.Thread(target = mqttc.loop_start)

device.connect()
thread1.start()
thread2.start()

print("Started Thread.")

try:
    while True:
        pass
except KeyboardInterrupt:
    print('KeyboardInterrupt!')
    device.disconnect()
    manager.quit()
    mqttc.loop_stop()
    thread1.join() 
    thread2.join() 
